/*
 * AR-3c realtime modbus driver - modbus.h
 *
 * Copyright (C) 2006  Krzysztof Mazur <krzysiek@podlesie.net>
 */

#ifndef MODBUS_H_INCLUDED
#define MODBUS_H_INCLUDED

#ifndef __GNUC__
#define __attribute__(x) /* attribute */
#endif

struct modbus_03_cmd {
	uint8_t busaddr;		/* slave address */
	uint8_t cmd;			/* command */
	uint8_t addr_hi;		/* address high byte */
	uint8_t addr_lo;		/* address low byte */
	uint8_t len_hi;			/* length high byte, words */
	uint8_t len_lo;			/* length low byte, words */
} __attribute__((packed));

struct modbus_03_resp {
	uint8_t busaddr;		/* slave address */
	uint8_t cmd;			/* command */
	uint8_t bytes;			/* big endian, payload bytes */
	uint16_t payload[0];
} __attribute__((packed));

/* 0x06 - Write Single Register */
struct modbus_06 {
	uint8_t busaddr;		/* slave address */
	uint8_t cmd;			/* command, 0x06 */
	uint8_t addr_hi;		/* address high byte */
	uint8_t addr_lo;		/* address low byte */
	uint16_t value;			/* value */
} __attribute__((packed));

/* 0x10 command - Write multiple registers */
struct modbus_10_cmd {
	uint8_t busaddr;		/* slave address */
	uint8_t cmd;			/* command */
	uint8_t addr_hi;		/* address high byte */
	uint8_t addr_lo;		/* address low byte */
	uint8_t len_hi;			/* length high byte, words */
	uint8_t len_lo;			/* length low byte, words */
	uint8_t bytes;			/* payload length */
	uint16_t payload[0];
} __attribute__((packed));

struct modbus_10_resp {
	uint8_t busaddr;		/* slave address */
	uint8_t cmd;			/* command */
	uint8_t addr_hi;		/* address high byte */
	uint8_t addr_lo;		/* address low byte */
	uint8_t len_hi;			/* length high byte, words */
	uint8_t len_lo;			/* length low byte, words */
} __attribute__((packed));

struct modbus_err_msg {
	uint8_t busaddr;		/* slave address */
	uint8_t cmd;			/* command */
	uint8_t err;			/* exception */
} __attribute__((packed));

#define MBEX_OK			0x00
#define MBEX_NOSYS		0x01
#define MBEX_FAULT		0x02
#define MBEX_INVAL		0x03
#define MBEX_SLAVE_FAIL		0x04
#define MBEX_ACK		0x05
#define MBEX_BUSY		0x06
#define MBEX_MEMPARERR		0x08
#define MBEX_NOGW		0x0a
#define MBEX_GWERR		0x0b

#endif
